//
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Flora License, Version 1.1 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://floralicense.org/license/
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an AS IS BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * Name        : DictionaryDataControlProvider
 * Version     : 1.0
 * Vendor      :
 * Description : This sample service application demonstrates how to use ISqlDataControlProviderEventListener and IMapDataControlProviderEventListener interfaces in Tizen::App namespace. @n
                 This service is used by DictionaryDataControl sample application.
 */

#include <FIo.h>
#include <FAppDataControlProviderManager.h>
#include "DictionaryDataControlProvider.h"

using namespace Tizen::Base;
using namespace Tizen::Base::Collection;
using namespace Tizen::Io;
using namespace Tizen::System;
using namespace Tizen::App;

static const wchar_t* PROVIDER_ID = L"http://tizen.org/datacontrol/provider/DictionaryDataControlProvider";

DictionaryDataControlProvider::DictionaryDataControlProvider(void)
{
}

DictionaryDataControlProvider::~DictionaryDataControlProvider(void)
{
}

ServiceApp*
DictionaryDataControlProvider::CreateInstance(void)
{
	// Create the instance through the constructor.
	return new (std::nothrow) DictionaryDataControlProvider();
}

bool
DictionaryDataControlProvider::OnAppInitializing(AppRegistry& appRegistry)
{
	result r = E_SUCCESS;
	bool ret = true;

	DataControlProviderManager* pDcMgr = DataControlProviderManager::GetInstance();
	TryReturn(pDcMgr != null, false, "Failed to get the data control provider manager instance.");

	r = pDcMgr->SetSqlDataControlProviderEventListener(this);
	TryReturn(!IsFailed(r), false, "Failed to Set the Sql data control provider listener.");

	r = pDcMgr->SetMapDataControlProviderEventListener(this);
	TryReturn(!IsFailed(r), false, "Failed to Set the Map data control provider listener.");

	Database* pDb = null;
	String sqlCommand(L"CREATE TABLE IF NOT EXISTS Dictionary ( WORD VARCHAR(30) PRIMARY KEY, WORD_DESC TEXT )");

	Registry* pReg = null;
	String secName(L"Dictionary");

	String dataPath = App::GetInstance()->GetAppDataPath();
	__dbPath.Append(String(dataPath + L"SampleDictionary.db"));
	__regPath.Append(String(dataPath + L"SampleDictionary.reg"));

	if (File::IsFileExist(__dbPath))
	{
		File::Remove(__dbPath);
	}

	pDb = new (std::nothrow) Database();
	TryCatch(pDb != null, ret = false, "new Database() is null");

	r = pDb->Construct(__dbPath, true);
	TryCatch(!IsFailed(r), ret = false, "Database Construct failed with error [%s] \n", GetErrorMessage(r));

	r = pDb->ExecuteSql(sqlCommand, true);
	TryCatch(!IsFailed(r), ret = false, "ExecuteSql failed with error [%s] \n", GetErrorMessage(r));

	if (File::IsFileExist(__regPath))
	{
		File::Remove(__regPath);
	}

	pReg = new (std::nothrow) Registry();
	TryCatch(pReg != null, ret = false, "new Registry() is null");

	r = pReg->Construct(__regPath, true);
	TryCatch(!IsFailed(r), ret = false, "Registry Construct failed with error [%s] \n", GetErrorMessage(r));

	r = pReg->AddSection(secName);
	TryCatch(!IsFailed(r), ret = false, "Registry AddSection failed with error [%s] \n", GetErrorMessage(r));

	// fall thru
CATCH:
	delete pDb;
	delete pReg;
	return ret;
}

bool
DictionaryDataControlProvider::OnAppInitialized(void)
{
	return true;
}

bool
DictionaryDataControlProvider::OnAppWillTerminate(void)
{
	return true;
}

bool
DictionaryDataControlProvider::OnAppTerminating(AppRegistry& appRegistry, bool forcedTermination)
{
	return true;
}

void
DictionaryDataControlProvider::OnLowMemory(void)
{
}

void
DictionaryDataControlProvider::OnBatteryLevelChanged(BatteryLevel batteryLevel)
{
}

void
DictionaryDataControlProvider::OnSqlDataControlInsertRequestReceived(RequestId reqId, const Tizen::Base::String& providerId,
																	 const Tizen::Base::String& dataId, const Tizen::Base::Collection::IMap& insertMap)
{
	Database* pDb = null;
	result r = E_SUCCESS;
	String errorMsg;
	bool providerResult = false;
	String sql;
	long long insertRowId = -1;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pDb = new (std::nothrow) Database();
			if (pDb == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] Db could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pDb->Construct(__dbPath, DB_OPEN_READ_WRITE, 0);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			sql = SqlStatementBuilder::CreateInsertStatement(dataId, insertMap);
			r = GetLastResult();
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"CreateInsertStatement failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			r = pDb->ExecuteSql(sql, true);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"ExecuteSql failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			
			insertRowId = pDb->GetLastInsertRowId();

			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pDb;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendSqlDataControlInsertResult(reqId, insertRowId);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("The data control provider failed to send the result.");
	}
}

void
DictionaryDataControlProvider::OnSqlDataControlSelectRequestReceived(RequestId reqId, const Tizen::Base::String& providerId,
																	 const Tizen::Base::String& dataId, const Tizen::Base::Collection::IList* pColumnList,
																	 const Tizen::Base::String* pWhere, const Tizen::Base::String* pOrder)
{
	Database* pDb = null;
	DbEnumerator* pEnum = null;
	result r = E_SUCCESS;
	bool providerResult = false;
	String errorMsg;
	String sql;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pDb = new (std::nothrow) Database();
			if (pDb == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] Db could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pDb->Construct(__dbPath, DB_OPEN_READ_ONLY, 0);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			sql = SqlStatementBuilder::CreateSelectStatement(dataId, pColumnList, pWhere, pOrder, null, null, null);
			r = GetLastResult();
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"CreateSelectStatement failed");
				goto CATCH;
			}

			AppLog("Select sql:%ls", sql.GetPointer());

			pEnum = pDb->QueryN(sql);
			r = GetLastResult();
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"QueryN failed");
				if (r == E_INVALID_OPERATION)
				{
					r = E_INVALID_ARG;
				}
				else
				{
					r = E_SYSTEM;
				}
				goto CATCH;
			}

			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendSqlDataControlSelectResult(reqId, pEnum);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("The data control provider failed to send the result.");
	}

	delete pEnum;
	delete pDb;

}

void
DictionaryDataControlProvider::OnSqlDataControlUpdateRequestReceived(RequestId reqId, const Tizen::Base::String& providerId,
																	 const Tizen::Base::String& dataId, const Tizen::Base::Collection::IMap& updateMap,
																	 const Tizen::Base::String* pWhere)
{
	Database* pDb = null;
	result r = E_SUCCESS;
	bool providerResult = false;
	String errorMsg;
	String sql;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pDb = new (std::nothrow) Database();
			if (pDb == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] Db could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pDb->Construct(__dbPath, DB_OPEN_READ_WRITE, 0);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			sql = SqlStatementBuilder::CreateUpdateStatement(dataId, updateMap, pWhere);
			r = GetLastResult();
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"CreateUpdateStatement failed");
				goto CATCH;
			}
			AppLog("Update sql:%ls", sql.GetPointer());

			r = pDb->ExecuteSql(sql, true);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database::ExecuteSql failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pDb;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendSqlDataControlUpdateDeleteResult(reqId);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}
	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}

}

void
DictionaryDataControlProvider::OnSqlDataControlDeleteRequestReceived(RequestId reqId, const Tizen::Base::String& providerId,
																	 const Tizen::Base::String& dataId, const Tizen::Base::String* pWhere)
{
	Database* pDb = null;
	result r = E_SUCCESS;
	String errorMsg;
	bool providerResult = false;
	String sql;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pDb = new (std::nothrow) Database();
			if (pDb == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] Db could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pDb->Construct(__dbPath, DB_OPEN_READ_WRITE, 0);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			sql = SqlStatementBuilder::CreateDeleteStatement(dataId, pWhere);
			r = GetLastResult();
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"CreateDeleteStatement failed");
				goto CATCH;
			}
			AppLog("Delete sql:%ls", sql.GetPointer());
			r = pDb->ExecuteSql(sql, true);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Database::ExecuteSql failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pDb;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendSqlDataControlUpdateDeleteResult(reqId);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}
}

void
DictionaryDataControlProvider::OnMapDataControlGetValueRequestReceived(RequestId reqId, const String& providerId,
																	   const String& dataId, const String& key)
{
	Registry* pReg = null;
	result r = E_SUCCESS;
	String retValue;
	ArrayList* pResultValueList = null;
	bool providerResult = false;
	String errorMsg;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pReg = new (std::nothrow) Registry();
			if (pReg == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] Registry could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pReg->Construct(__regPath, false);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			r = pReg->GetValue(dataId, key, retValue);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::GetValue failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			AppLog("retValue(%ls) \n", retValue.GetPointer());

			pResultValueList = new (std::nothrow) ArrayList;
			if (pResultValueList == null)
			{
				AppLog("new ArrayList() is null");
				errorMsg.Append(L"[E_OUT_OF_MEMORY] new ArrayList() is null");
				goto CATCH;
			}
			pResultValueList->Add(*new String(retValue));

			providerResult = true;

		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pReg;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendMapDataControlResult(reqId, pResultValueList);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}

	if (pResultValueList)
	{
		pResultValueList->RemoveAll(true);
		delete pResultValueList;
	}

}

void
DictionaryDataControlProvider::OnMapDataControlAddValueRequestReceived(RequestId reqId, const String& providerId,
																	   const String& dataId, const String& key, const String& value)
{
	Registry* pReg = null;
	result r = E_SUCCESS;
	String errorMsg;
	bool providerResult = false;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pReg = new (std::nothrow) Registry();
			if (pReg == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] registry could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pReg->Construct(__regPath, false);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			r = pReg->AddValue(dataId, key, value);
			if (IsFailed(r))
			{
				if (r != E_KEY_ALREADY_EXIST)
				{
					errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
					errorMsg.Append(L"Registry::AddValue failed");
					r = E_SYSTEM;
					goto CATCH;
				}
			}
			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pReg;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendMapDataControlResult(reqId, null);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}
}

void
DictionaryDataControlProvider::OnMapDataControlSetValueRequestReceived(RequestId reqId, const String& providerId,
																	   const String& dataId, const String& key, const String& oldValue, const String& newValue)
{
	Registry* pReg = null;
	result r = E_SUCCESS;
	String errorMsg;
	bool providerResult = false;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pReg = new (std::nothrow) Registry();
			if (pReg == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] registry could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pReg->Construct(__regPath, false);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			r = pReg->SetValue(dataId, key, newValue);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::SetValue failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pReg;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendMapDataControlResult(reqId, null);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}

	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}
}

void
DictionaryDataControlProvider::OnMapDataControlRemoveValueRequestReceived(RequestId reqId, const String& providerId,
																		  const String& dataId, const String& key, const String& value)
{
	Registry* pReg = null;
	result r = E_SUCCESS;
	String errorMsg;
	bool providerResult = false;

	if (providerId.CompareTo(String(PROVIDER_ID)) == 0)
	{
		if (dataId.CompareTo(L"Dictionary") == 0)
		{
			pReg = new (std::nothrow) Registry();
			if (pReg == null)
			{
				errorMsg.Append(L"[E_OUT_OF_MEMORY] registry could not be instantiated. Insufficient memory");
				r = E_OUT_OF_MEMORY;
				goto CATCH;
			}

			r = pReg->Construct(__regPath, false);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::Construct failed");
				r = E_SYSTEM;
				goto CATCH;
			}

			r = pReg->RemoveValue(dataId, key);
			if (IsFailed(r))
			{
				errorMsg.Format(32, L"[%s]", GetErrorMessage(r));
				errorMsg.Append(L"Registry::RemoveValue failed");
				r = E_SYSTEM;
				goto CATCH;
			}
			providerResult = true;
		}
		else
		{
			errorMsg.Append(L"[E_INVALID_ARG] Invalid dataId was passed");
			r = E_INVALID_ARG;
		}
	}
	else
	{
		errorMsg.Append(L"[E_INVALID_ARG] Invalid ProviderId was passed");
		r = E_INVALID_ARG;
	}

	// fall thru
CATCH:
	delete pReg;

	if (providerResult == true)
	{
		r = DataControlProviderManager::GetInstance()->SendMapDataControlResult(reqId, null);
	}
	else
	{
		AppLog("errorMsg: %ls", errorMsg.GetPointer());
		r = DataControlProviderManager::GetInstance()->SendDataControlError(reqId, errorMsg);
	}
	if (IsFailed(r))
	{
		AppLog("[%s]The data control provider failed to send the result.", GetErrorMessage(r));
	}
}
